function gadget:GetInfo()
  return {
    name      = "Dynamic collision volume & Hitsphere Scaledown",
    desc      = "Adjusts collision volume for pop-up style units & Reduces the diameter of default sphere collision volume for 3DO models",
    author    = "Deadnight Warrior",
    date      = "Sep 30, 2010",
    license   = "GNU GPL, v2 or later",
    layer     = 0,
    enabled   = true  --  loaded by default?
  }
end

--[[  from Spring Wiki, info about CollisionVolumeData
Spring.GetUnitCollisionVolumeData ( number unitID ) -> 
	number scaleX, number scaleY, number scaleZ, number offsetX, number offsetY, number offsetZ,
	number volumeType, number testType, number primaryAxis, boolean disabled

Spring.SetUnitCollisionVolumeData ( number unitID, number scaleX, number scaleY, number scaleZ,
					number offsetX, number offsetY, number offsetX,
					number vType, number tType, number Axis ) -> nil

   possible vType constants
     DISABLED = -1  disables collision volume and collision detection for that unit, do not use
     ELLIPSOID = 0
     CYLINDER =  1
     BOX =       2
     SPHERE =    3
     FOOTPRINT = 4  intersection of sphere and footprint-prism, makes a sphere collision volume, default
	 
   possible tType constants, for non-sphere collision volumes use 1
     COLVOL_TEST_DISC = 0
     COLVOL_TEST_CONT = 1

   possible Axis constants, use non-zero only for Cylinder test
     COLVOL_AXIS_X = 0
     COLVOL_AXIS_Y = 1
     COLVOL_AXIS_Z = 2
]]--


local popupUnits = {}		--list of pop-up style units
local unitCollisionVolume = {}	--pop-up style unit collision volume definitions, ones entered here are for XTA, for other mods modify apropriatly
	
	unitCollisionVolume["arm_advanced_radar_tower"] = {
		on={60,80,60,0,15,0,0,1,0},    --{Xscale,Yscale,Zscale, Xoffset,Yoffset,Zoffset, vType,tType,axis}
		off={32,48,32,0,-10,0,0,1,0},
	}
	unitCollisionVolume["arm_advanced_sonar_station"] = {
		on={63,70,63,0,-7,0,0,1,0},
		off={24,40,24,0,-5,0,0,1,0},
	}
	unitCollisionVolume["arm_ambusher"] = {
		on={49,45,49,-0.5,-14,0,0,1,0},
		off={49,26,49,-0.5,-14,0,0,1,0},
	}
	unitCollisionVolume["arm_annihilator"] = {
		on={50,55,50,0,10,0,2,1,0},
		off={50,35,50,0,0,0,2,1,0},
	}
	unitCollisionVolume["arm_moho_metal_maker"] = {
		on={60,78,60,0,10,0,1,1,1},
		off={60,50,60,0,-4,0,1,1,1},
	}
	unitCollisionVolume["arm_seaplane_platform"] = {
		on={105,66,105,0,33,0,2,1,0},
		off={105,44,105,0,0,0,2,1,0},
	}
	unitCollisionVolume["arm_solar_collector"] = {
		on={83,78,83,0,-18,1,0,1,0},
		off={50,78,50,0,-18,1,0,1,0},
	}
	unitCollisionVolume["arm_targeting_facility"] = {
		on={62,34,62,0,0,0,2,1,0},
		off={55,78,55,0,-19.5,0,0,1,0},
	}
	unitCollisionVolume["core_doomsday_machine"] = {
		on={55,112,55,0,-3,0,2,1,0},
		off={48,86,48,0,-15,0,2,1,0},
	}
	unitCollisionVolume["core_moho_metal_maker"] = {
		on={60,60,60,0,0,0,1,1,1},
		off={50,92,50,0,-22.5,0,0,1,0},
	}
	unitCollisionVolume["core_seaplane_platform"] = {
		on={112,60,112,0,30,0,1,1,1},
		off={112,40,112,0,0,0,1,1,1},
	}
	unitCollisionVolume["core_solar_collector"] = {
		on={86,78,86,0,-18,0,0,1,0},
		off={77,78,77,0,-28,0,0,1,0},
	}
	unitCollisionVolume["core_targeting_facility"] = {
		on={64,20,64,0,0,0,1,1,1},
		off={38,20,38,0,0,0,1,1,1},
	}
	unitCollisionVolume["core_toaster"] = {
		on={44,23,44,0,0,0,2,1,0},
		off={44,8,44,0,-9,0,2,1,0},
	}
	unitCollisionVolume["core_viper"] = {
		on={67,136,67,0,-21,0,0,1,0},
		off={51,20,53,0,-12,0,2,1,0},
	}


if (gadgetHandler:IsSyncedCode()) then

	--Reduces the diameter of default (unspecified) collision volume for 3DO models,
	--for S3O models it's not needed and will in fact result in wrong collision volume
	function gadget:UnitCreated(unitID, unitDefID, unitTeam)
		if UnitDefs[unitDefID].model.type=="3do" then
			local xs, ys, zs, xo, yo, zo, vtype, htype, axis, _ = Spring.GetUnitCollisionVolumeData(unitID)
			if (vtype==4 and xs==ys and ys==zs) then
				if (xs>47 and not UnitDefs[unitDefID].canFly) then
					Spring.SetUnitCollisionVolumeData(unitID, xs*0.68, ys*0.68, zs*0.68,  xo, yo, zo,  vtype, htype, axis)
				else
					Spring.SetUnitCollisionVolumeData(unitID, xs*0.75, ys*0.75, zs*0.75,  xo, yo, zo,  vtype, htype, axis)
				end
		    end
		end
	end


	--[[ unsupported by engine ATM, same as for 3DO units, but for features
	function gadget:FeatureCreated(featureID, allyTeam)
		local xs, ys, zs, xo, yo, zo, vtype, htype, axis, _ = Spring.GetFeatureCollisionVolumeData(featureID)
		if (vtype==4 and xs==ys and ys==zs) then
			Spring.SetFeatureCollisionVolumeData(unitID, xs*0.75, ys*0.75, zs*0.75,  xo, yo, zo,  vtype, htype, axis)

		end
	end
	]]--


	--check if a unit is pop-up type (the list must be entered manually)
	function gadget:UnitFinished(unitID, unitDefID, unitTeam)
		local un = UnitDefs[unitDefID].name
		if unitCollisionVolume[un] then
			popupUnits[unitID]=un
		end
	end


	--check if a pop-up type unit was destroyed
	function gadget:UnitDestroyed(unitID, unitDefID, unitTeam)
		if popupUnits[unitID] then
			popupUnits[unitID] = nil
		end
	end

	
	--Dynamic adjustment of pop-up style of units' collision volumes based on
	--unit's ARMORED status, runs twice per second
	function gadget:GameFrame(n)
		if (n%15 ~= 0) then
			return
		end
		local p
		for unitID,name in pairs(popupUnits) do
			if Spring.GetUnitArmored(unitID) then
				p = unitCollisionVolume[name].off
			else
				p = unitCollisionVolume[name].on

				--[[ if units doesn't use ARMORED variable, uncomment this block to make it use ACTIVATION as well
				if ( Spring.GetUnitIsActive(unitID) ) then
					p = unitCollisionVolume[name].on
				else
					p = unitCollisionVolume[name].off
				end
				]]--

			end
			Spring.SetUnitCollisionVolumeData(unitID, p[1], p[2], p[3], p[4], p[5], p[6], p[7], p[8], p[9])
		end
	end
end