--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
--  author:  jK
--
--  Copyright (C) 2007,2008.
--  Licensed under the terms of the GNU GPL, v2 or later.
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

function widget:GetInfo()
  return {
    name      = "LupsManager",
    desc      = "",
    author    = "jK",
    date      = "Feb, 2008",
    license   = "GNU GPL, v2 or later",
    layer     = 10,
    enabled   = true,
    handler   = true,
  }
end


include("Configs/lupsFXs.lua")

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local function MergeTable(table1,table2)
  local result = {}
  for i,v in pairs(table2) do 
    if (type(v)=='table') then
      result[i] = MergeTable(v,{})
    else
      result[i] = v
    end
  end
  for i,v in pairs(table1) do 
    if (result[i]==nil) then
      if (type(v)=='table') then
        if (type(result[i])~='table') then result[i] = {} end
        result[i] = MergeTable(v,result[i])
      else
        result[i] = v
      end
    end
  end
  return result
end


local function blendColor(c1,c2,mix)
  if (mix>1) then mix=1 end
  local mixInv = 1-mix
  return {
    c1[1]*mixInv + c2[1]*mix,
    c1[2]*mixInv + c2[2]*mix,
    c1[3]*mixInv + c2[3]*mix,
    (c1[4] or 1)*mixInv + (c2[4] or 1)*mix
  }
end


local function blend(a,b,mix)
  if (mix>1) then mix=1 end
  return a*(1-mix) + b*mix
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local UnitEffects = {

  --// ENERGY STORAGE //--------------------
  [UnitDefNames["core_energy_storage"].id] = {
    {class='GroundFlash',options=groundFlashCorestor},
  },
  [UnitDefNames["arm_energy_storage"].id] = {
    {class='GroundFlash',options=groundFlashArmestor},
  },

  --// PLANES still need to do work here //----------------------------
  [UnitDefNames["arm_freedom_fighter"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=6, length=45, piece="rearthrust", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=6, length=45, piece="rearthrust2", onActive=true}},
 },
  [UnitDefNames["arm_tornado"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=6, length=45, piece="thrust", onActive=true}},
 },
  [UnitDefNames["arm_albatross"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=6, length=45, piece="thrust", onActive=true}},
 },
  [UnitDefNames["arm_hawk"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=6, length=75, piece="rearthrust", onActive=true}},
  },
  [UnitDefNames["core_fink"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=3, length=35, piece="thrustb", onActive=true}},
  },
  [UnitDefNames["core_titan"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=5, length=65, piece="thrustb", onActive=true}},
  },
  [UnitDefNames["arm_lancet"].id] = {
   {class='AirJet',options={color={0.3,0.1,0}, width=5, length=65, piece="thrust", onActive=true}},
  },
  [UnitDefNames["core_avenger"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=2.4, length=42, piece="thrust1", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=2.4, length=42, piece="thrust2", onActive=true}},
  },
  [UnitDefNames["core_typhoon"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=1.1, length=31, piece="thrustl1", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=1.8, length=47, piece="thrustl2", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=1.8, length=47, piece="thrustl3", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=1.1, length=31, piece="thrustr1", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=1.8, length=47, piece="thrustr2", onActive=true}},
    {class='AirJet',options={color={0.3,0.1,0}, width=1.8, length=47, piece="thrustr3", onActive=true}},
  },
  [UnitDefNames["core_shadow"].id] = {
    {class='AirJet',options={color={0.6,0.1,0}, width=3.5, length=52, piece="thrust1", onActive=true}},
    {class='AirJet',options={color={0.6,0.1,0}, width=3.5, length=52, piece="thrust2", onActive=true}},
  },
  [UnitDefNames["arm_thunder"].id] = {
    {class='ThundAirJet',options={color={0.3,0.1,0}, width=2.9, length=52, piece="thrust1", onActive=true}},
    {class='ThundAirJet',options={color={0.3,0.1,0}, width=2.9, length=52, piece="thrust2", onActive=true}},
    {class='ThundAirJet',options={color={0.3,0.1,0}, width=2.9, length=52, piece="thrust3", onActive=true}},
    {class='ThundAirJet',options={color={0.3,0.1,0}, width=2.9, length=52, piece="thrust4", onActive=true}},
  },
  [UnitDefNames["core_hurricane"].id] = {
    {class='AirJet',options={color={0.9,0.3,0}, width=10, length=80, piece="thrust", onActive=true}},
  },
  [UnitDefNames["arm_phoenix"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=8, length=75, piece="thrust", onActive=true}},
  },
  [UnitDefNames["core_vamp"].id] = {
    {class='AirJet',options={color={0.6,0.1,0}, width=3.5, length=65, piece="thrustb", onActive=true}},
  },
  [UnitDefNames["core_vulture"].id] = {
    {class='AirJet',options={color={0.8,0.2,0}, width=4, length=50, piece="thrust", onActive=true}},
  },
  [UnitDefNames["core_hunter"].id] = {
    {class='AirJet',options={color={0.8,0.2,0}, width=4, length=50, piece="thrust", onActive=true}},
  },
  [UnitDefNames["arm_eagle"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=3.5, length=50, piece="thrust", onActive=true}},
  },
  [UnitDefNames["arm_seahawk"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=4, length=50, piece="thrust", onActive=true}},
  },
  [UnitDefNames["core_silhouette"].id] = {
    {class='AirJet',options={color={0.4,0.1,0}, width=1.8, length=52, piece="thrust1a", onActive=true}},
    {class='AirJet',options={color={0.4,0.1,0}, width=1.8, length=52, piece="thrust1b", onActive=true}},
    {class='AirJet',options={color={0.6,0.1,0}, width=3.7, length=76, piece="thrust2", onActive=true}},
  },
  [UnitDefNames["arm_harpoon"].id] = {
    {class='AirJet',options={color={0.3,0.1,0}, width=4.5, length=70, piece="thrust", onActive=true}},
  },
}

local abs = math.abs
local spGetSpectatingState = Spring.GetSpectatingState
local spGetUnitDefID       = Spring.GetUnitDefID
local spGetUnitRulesParam  = Spring.GetUnitRulesParam
local spGetUnitIsActive    = Spring.GetUnitIsActive

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local Lups  -- Lua Particle System
local LupsAddFX
local particleIDs = {}
local initialized = false --// if LUPS isn't started yet, we try it once a gameframe later
local tryloading  = 1     --// try to activate lups if it isn't found

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local function ClearFxs(unitID)
  if (particleIDs[unitID]) then
    for _,fxID in ipairs(particleIDs[unitID]) do
      Lups.RemoveParticles(fxID)
    end
    particleIDs[unitID] = nil
  end
end

local function ClearFx(unitID, fxIDtoDel)
  if (particleIDs[unitID]) then
	local newTable = {}
	for _,fxID in ipairs(particleIDs[unitID]) do
		if fxID == fxIDtoDel then 
			Lups.RemoveParticles(fxID)
		else 
			newTable[#newTable+1] = fxID
		end
    end
	if #newTable == 0 then 
		particleIDs[unitID] = nil
	else 
		particleIDs[unitID] = newTable
	end
  end
end

local function AddFxs(unitID,fxID)
  if (not particleIDs[unitID]) then
    particleIDs[unitID] = {}
  end

  local unitFXs = particleIDs[unitID]
  unitFXs[#unitFXs+1] = fxID
end
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local function UnitFinished(_,unitID,unitDefID)

  local effects = UnitEffects[unitDefID]
  if (effects) then
    for _,fx in ipairs(effects) do
      if (not fx.options) then
        Spring.Echo("LUPS DEBUG ", UnitDefs[unitDefID].name, fx and fx.class)
        return
      end

      if (fx.class=="GroundFlash") then
        fx.options.pos = { Spring.GetUnitBasePosition(unitID) }
      end
      fx.options.unit = unitID
      AddFxs( unitID,LupsAddFX(fx.class,fx.options) )
      fx.options.unit = nil
    end
  end
end

local function UnitDestroyed(_,unitID,unitDefID)
  ClearFxs(unitID)
end


local function UnitEnteredLos(_,unitID)
  local spec, fullSpec = spGetSpectatingState()
  if (spec and fullSpec) then return end
    
  local unitDefID = spGetUnitDefID(unitID)
  local effects   = UnitEffects[unitDefID]
  if (effects) then
	for _,fx in ipairs(effects) do
	  if (fx.options.onActive == true) and (spGetUnitIsActive(unitID) == nil) then
		break
	  else
		if (fx.class=="GroundFlash") then
		  fx.options.pos = { Spring.GetUnitBasePosition(unitID) }
		end
		fx.options.unit = unitID
		fx.options.under_construction = spGetUnitRulesParam(unitID, "under_construction")
		AddFxs( unitID,LupsAddFX(fx.class,fx.options) )
		fx.options.unit = nil
	  end
	end
  end
  
end


local function UnitLeftLos(_,unitID)
  local spec, fullSpec = spGetSpectatingState()
  if (spec and fullSpec) then return end

  ClearFxs(unitID)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local function PlayerChanged(_,playerID)
  if (playerID == Spring.GetMyPlayerID()) then
    --// clear all FXs
    for _,unitFxIDs in pairs(particleIDs) do
      for _,fxID in ipairs(unitFxIDs) do
        Lups.RemoveParticles(fxID)
      end
    end
    particleIDs = {}

    widgetHandler:UpdateWidgetCallIn("Update",widget)
  end
end

local function CheckForExistingUnits()
  --// initialize effects for existing units
  local allUnits = Spring.GetAllUnits();
  for i=1,#allUnits do
    local unitID    = allUnits[i]
    local unitDefID = Spring.GetUnitDefID(unitID)
    if (spGetUnitRulesParam(unitID, "under_construction") ~= 1) then
		UnitFinished(nil,unitID,unitDefID)
	end
  end

  widgetHandler:RemoveWidgetCallIn("Update",widget)
end

function widget:GameFrame()
  if (Spring.GetGameFrame() > 0) then
    Spring.SendLuaRulesMsg("lups running","allies")
    widgetHandler:RemoveWidgetCallIn("GameFrame",widget)
  end
end

function widget:Update()
  Lups = WG['Lups']
  local LupsWidget = widgetHandler.knownWidgets['Lups'] or {}

  --// Lups running?
  if (not initialized) then
    if (Lups and LupsWidget.active) then
      if (tryloading==-1) then
        Spring.Echo("LuaParticleSystem (Lups) activated.")
      end
      initialized=true
      return
    else
      if (tryloading==1) then
        Spring.Echo("Lups not found! Trying to activate it.")
        widgetHandler:EnableWidget("Lups")
        tryloading=-1
        return
      else
        Spring.Echo("LuaParticleSystem (Lups) couldn't be loaded!")
        widgetHandler:RemoveWidgetCallIn("Update",self)
        return
      end
    end
  end

  LupsAddFX = Lups.AddParticles

  Spring.SendLuaRulesMsg("lups running","allies")

  widget.UnitFinished   = UnitFinished
  widget.UnitDestroyed  = UnitDestroyed
  widget.UnitEnteredLos = UnitEnteredLos
  widget.UnitLeftLos    = UnitLeftLos
  widget.GameFrame      = GameFrame
  widget.PlayerChanged  = PlayerChanged
  widgetHandler:UpdateWidgetCallIn("UnitFinished",widget)
  widgetHandler:UpdateWidgetCallIn("UnitDestroyed",widget)
  widgetHandler:UpdateWidgetCallIn("UnitEnteredLos",widget)
  widgetHandler:UpdateWidgetCallIn("UnitLeftLos",widget)
  widgetHandler:UpdateWidgetCallIn("GameFrame",widget)
  widgetHandler:UpdateWidgetCallIn("PlayerChanged",widget)

  widget.Update = CheckForExistingUnits
  widgetHandler:UpdateWidgetCallIn("Update",widget)
end

function widget:Shutdown()
  if (initialized) then
    for _,unitFxIDs in pairs(particleIDs) do
      for _,fxID in ipairs(unitFxIDs) do
        Lups.RemoveParticles(fxID)
      end
    end
    particleIDs = {}
  end

  Spring.SendLuaRulesMsg("lups shutdown","allies")
end
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------