--------------------------------------------------------------------------------
-- Unit Definitions Post-processing
--------------------------------------------------------------------------------

local modOptions
if (Spring.GetModOptions) then
  modOptions = Spring.GetModOptions()

local function tobool(val)
  local t = type(val)
  if (t == 'nil') then
    return false
  elseif (t == 'boolean') then
    return val
  elseif (t == 'number') then
    return (val ~= 0)
  elseif (t == 'string') then
    return ((val ~= '0') and (val ~= 'false'))
  end
  return false
end

--------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Starting resource storage adjustment

if (modOptions) then
   local stEn = modOptions.startenergy or 1000
   local stMe = modOptions.startmetal or 1000
   for name, ud in pairs(UnitDefs) do  
	if (ud.unitname == "arm_commander" or ud.unitname == "core_commander") then
		ud.energyStorage = stEn + 50
		ud.metalStorage = stMe + 50
	elseif (ud.unitname == "arm_u0commander" or ud.unitname == "core_u0commander") then
		ud.energyStorage = stEn + 50
		ud.metalStorage = stMe + 50
	elseif (ud.unitname == "arm_ucommander" or ud.unitname == "core_ucommander") then
		ud.energyStorage = stEn + 1000
		ud.metalStorage = stMe + 1000
	elseif (ud.unitname == "arm_u2commander" or ud.unitname == "core_u2commander") then
		ud.energyStorage = stEn + 1500
		ud.metalStorage = stMe + 1500
	elseif (ud.unitname == "arm_u3commander" or ud.unitname == "core_u3commander") then
		ud.energyStorage = stEn + 4000
		ud.metalStorage = stMe + 4000
	elseif (ud.unitname == "arm_u4commander" or ud.unitname == "core_u4commander") then
		ud.energyStorage = stEn + 9000
		ud.metalStorage = stMe + 9000
	elseif (ud.unitname == "armcom" or ud.unitname == "corcom") then
		ud.energyStorage = stEn + 200
		ud.metalStorage = stMe
	elseif (ud.unitname == "arm_decoy_commander" or ud.unitname == "core_decoy_commander") then
		ud.energyStorage = stEn
		ud.metalStorage = stMe
	elseif (ud.unitname == "arm_base" or ud.unitname == "core_base") then
		ud.energyStorage = stEn
		ud.metalStorage = stMe
	elseif (ud.unitname == "arm_nincommander" or ud.unitname == "core_nincommander") then
		ud.energyStorage = stEn + 50
		ud.metalStorage = stMe + 9000
	end
   end
end
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Metal Extraction Bonus

if (modOptions and modOptions.metalmult) then
  for name in pairs(UnitDefs) do
    local em = UnitDefs[name].extractsmetal
    if (em) then
      UnitDefs[name].extractsmetal = em * modOptions.metalmult
    end
  end
end
--------------------------------------------------------------------------------
-- Hitpoint Bonus

if (modOptions and modOptions.hitmult) then
  for name in pairs(UnitDefs) do
    local em = UnitDefs[name].maxdamage
    if (em) then
      UnitDefs[name].maxdamage = em * modOptions.hitmult 
    end
  end
end
--------------------------------------------------------------------------------
-- Velocity Bonus

if (modOptions and modOptions.velocitymult) then
  for name in pairs(UnitDefs) do
    local em = UnitDefs[name].maxvelocity
    if (em) then
      UnitDefs[name].maxvelocity = em * modOptions.velocitymult
    end
  end
end
--------------------------------------------------------------------------------
-- Build Bonus
--

if (modOptions and modOptions.workermult) then
  for name in pairs(UnitDefs) do
    local em = UnitDefs[name].workertime
    if (em) then
      UnitDefs[name].workertime = em * modOptions.workermult 
    end
  end
end
--------------------------------------------------------------------------------
-- Energy Production Bonus
--

if (modOptions and modOptions.energymult) then
  for name in pairs(UnitDefs) do
    local em = UnitDefs[name].energymake
    if (em) then
      UnitDefs[name].energymake = em * modOptions.energymult
    end
  end
end

if (modOptions and modOptions.energymult) then
  for name in pairs(UnitDefs) do
    local em = UnitDefs[name].totalEnergyOut
    if (em) then
      UnitDefs[name].totalEnergyOut = em * modOptions.energymult
    end
  end
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
local function disableunits(unitlist)
  for name, ud in pairs(UnitDefs) do
    if (ud.buildoptions) then
      for _, toremovename in ipairs(unitlist) do
        for index, unitname in pairs(ud.buildoptions) do
          if (unitname == toremovename) then
            table.remove(ud.buildoptions, index)
          end
        end
      end
    end
  end
end
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- Disable all XTA units since 8.1?
-- 

if (modOptions and tobool(modOptions.newunits)) then
  disableunits({
	"arm_floating_radar", "arm_vanguard", "arm_daybreak",
	"arm_nano_tower", "arm_repulsor", "arm_spidey",
	"arm_sub_pen", "arm_floating_light_laser_tower",
	"arm_ornith", "arm_underwater_moho_mine",

	"core_floating_radar", "core_viking", "core_nightfall",
	"core_nano_tower", "core_resistor", "core_tic",
	"core_sub_pen", "core_floating_light_laser_tower",
	"core_underwater_moho_mine", "core_zeppelin"
  })
end
--------------------------------------------------------------------------------
-- Disable XTAIDS unitpack
--

if (modOptions and not tobool(modOptions.xtaidunits)) then
  disableunits({
	"yeomen", "armona",  "arm_evaperator", "paci",
	"armsparkle", "neutral_eyes", "walker", "armpur",
	"armbfortew", "armarch", "armfff", "mercury",
	"armbugger", "tadg07", "armraz", "armhyren",
	"demolish", "armrebel",  "boa",  "arm_tor",
	"tadg05", "armraz", "arm_infinite",

	"corbyp", "justice", "corpur", "astank", "corjar",
	"corscrew", "corcrw", "corfff", "screamer",
	"corebfortns", "corhback", "commando", "corevashp",
	"core_emperor", "core_egg_shell", "cor_jumbo",
	"scorp3g", "nsaclash", "tadg06"
  })
end

end